#version 300 es

//USE_LIGHT#define USE_LIGHT
//USE_SHADOW#define USE_SHADOW

precision lowp float;

#ifdef USE_SHADOW
uniform lowp sampler2DShadow u_shadowTexture;
uniform lowp vec2 u_shadowTextureSize;
#endif

#ifdef USE_LIGHT
uniform float u_shadowVolume;
#endif

in lowp vec4 v_color;

#ifdef USE_SHADOW
in mediump vec3  v_shadow;
#endif

#ifdef USE_LIGHT
in mediump float v_light;
#endif

out vec4 fragmentColor;

void main()
{
    fragmentColor = v_color;

#ifdef USE_SHADOW
    if (v_light > 0.99)
    {
        fragmentColor.rgb *= 1. - u_shadowVolume;
        return;
    }
    
    vec3 v = v_shadow;
    v.z = clamp(v.z, 0., 1.);
    
    mediump float s = texture(u_shadowTexture, v);
    if (s == 0.)
    {
        vec2 sm = fract(v.xy * u_shadowTextureSize);
        if (sm.x > sm.y)
        {
            s  = textureOffset(u_shadowTexture, v, ivec2( 1,  0));
            s += textureOffset(u_shadowTexture, v, ivec2( 0, -1));
        }
        else
        {
            s  = textureOffset(u_shadowTexture, v, ivec2( -1, 0));
            s += textureOffset(u_shadowTexture, v, ivec2( 0,  1));
        }
        s = float(s == 2.f);
        
        if (s == 0.)
        {
            if (sm.x + sm.y < 1.)
            {
                s  = textureOffset(u_shadowTexture, v, ivec2( -1, 0));
                s += textureOffset(u_shadowTexture, v, ivec2( 0, -1));
            }
            else
            {
                s  = textureOffset(u_shadowTexture, v, ivec2( 1,  0));
                s += textureOffset(u_shadowTexture, v, ivec2( 0,  1));
            }
            s = float(s == 2.f);
        }
    }
    
    fragmentColor.rgb *= 1. - u_shadowVolume * max(v_light, 1. - s);
#else
#ifdef USE_LIGHT
    fragmentColor.rgb *= 1. - u_shadowVolume * v_light;
#endif
#endif
}
