#version 300 es

//TYPE_TEXTURE#define TYPE_TEXTURE
//TYPE_COLOR#define TYPE_COLOR
//TYPE_MONO#define TYPE_MONO
//USE_LIGHT#define USE_LIGHT
//USE_SHADOW#define USE_SHADOW
//VOXEL#define VOXEL
//ANDROID#define ANDROID

// uniforms
#ifdef ANDROID
uniform vec4 u_mats[/U_MATS_SIZEx4/];
#else
uniform mat4 u_mats[/U_MATS_SIZE/];
#endif
#ifdef USE_LIGHT
uniform vec4 u_lightVecs[/U_LIGHTVECS_SIZE/];
#endif
#ifdef USE_SHADOW
uniform mat4 u_shadowMat;
#endif

// attributes
layout(location = 0) in vec4  a_position_i;
#ifdef TYPE_COLOR
layout(location = 1) in vec4  a_color;
#endif
#ifdef TYPE_TEXTURE
layout(location = 2) in vec2  a_uv;
#endif
#ifdef USE_LIGHT
#ifdef VOXEL
layout(location = 3) in float a_normal;
#else
layout(location = 3) in vec3  a_normal;
#endif
#endif

// varying
#ifdef TYPE_TEXTURE
out vec2 v_uv;
#endif
#ifdef TYPE_COLOR
out vec4 v_color;
#endif
#ifdef USE_LIGHT
out mediump float v_light;
#endif
#ifdef USE_SHADOW
out mediump vec3 v_shadow;
#endif

void main()
{
    int i       = int(a_position_i.w);
#ifdef ANDROID
    int j       = i << 2;
    gl_Position = mat4(u_mats[j++], u_mats[j++], u_mats[j++], u_mats[j]) * vec4(a_position_i.xyz, 1.);
#else
    gl_Position = u_mats[i] * vec4(a_position_i.xyz, 1.);
#endif
    
#ifdef TYPE_TEXTURE
    v_uv = a_uv;
#endif

#ifdef TYPE_COLOR
    v_color = a_color;
#endif

#ifdef USE_LIGHT
#ifdef VOXEL
    int n   = int(a_normal);
#ifdef ANDROID
    float s = float(1 - ((n&1)<<1));
    n >>= 1;
    if      (n == 0) v_light = u_lightVecs[i].x;
    else if (n == 1) v_light = u_lightVecs[i].y;
    else if (n == 2) v_light = u_lightVecs[i].z;
    v_light = 1. - max(v_light * s, 0.);
#else
    v_light = 1. - max(u_lightVecs[i][n>>1] * float(1 - ((n&1)<<1)), 0.);
#endif
#else
    mediump float l = (clamp(dot(u_lightVecs[i].xyz, a_normal), 0.25, 0.75) - 0.25) * 1.33333;
    v_light = 1. - (-l * (l - 2.));
#endif
#endif

#ifdef USE_SHADOW
    vec4 position = u_shadowMat * gl_Position;
    v_shadow      = position.xyz / position.w;
#endif
}
