#version 300 es

//TYPE_TEXTURE#define TYPE_TEXTURE
//TYPE_COLOR#define TYPE_COLOR
//TYPE_MONO#define TYPE_MONO
//USE_LIGHT#define USE_LIGHT
//USE_SHADOW#define USE_SHADOW
//VOXEL#define VOXEL
//ANDROID#define ANDROID
//ATTRIB_MATRIX#define ATTRIB_MATRIX

// uniforms
#ifdef ANDROID
uniform vec4 u_mats[/U_MATS_SIZEx4/];
#else
uniform mat4 u_mats[/U_MATS_SIZE/];
#endif
uniform int  u_umatsPI;
#ifdef USE_LIGHT
uniform vec4 u_lightVecs[/U_LIGHTVECS_SIZE/];
uniform int  u_matsPI;
#endif
#ifdef USE_SHADOW
uniform mat4 u_shadowMat;
#endif

// attributes
layout(location =  0) in vec4 a_position_i;
#ifdef TYPE_COLOR
layout(location =  1) in vec4 a_color;
#endif
#ifdef TYPE_TEXTURE
layout(location =  2) in vec2 a_uv;
#endif
#ifdef USE_LIGHT
#ifdef VOXEL
layout(location =  3) in float a_normal;
#else
layout(location =  3) in vec3 a_normal;
#endif
#endif
#ifdef ATTRIB_MATRIX
layout(location =  4) in mat4 a_mat1;
layout(location =  8) in mat4 a_mat2;
layout(location = 12) in mat4 a_mat3;
#else
layout(location =  4) in vec4 a_mat1_1;
layout(location =  5) in vec4 a_mat1_2;
layout(location =  6) in vec4 a_mat1_3;
layout(location =  7) in vec4 a_mat1_4;
layout(location =  8) in vec4 a_mat2_1;
layout(location =  9) in vec4 a_mat2_2;
layout(location = 10) in vec4 a_mat2_3;
layout(location = 11) in vec4 a_mat2_4;
layout(location = 12) in vec4 a_mat3_1;
layout(location = 13) in vec4 a_mat3_2;
layout(location = 14) in vec4 a_mat3_3;
layout(location = 15) in vec4 a_mat3_4;
#endif

// varying
flat out int v_instanceID;
#ifdef TYPE_TEXTURE
out vec2 v_uv;
#endif
#ifdef TYPE_COLOR
out vec4 v_color;
#endif
#ifdef USE_LIGHT
out mediump float v_light;
#endif
#ifdef USE_SHADOW
out mediump vec3 v_shadow;
#endif

void main()
{
    vec4 position = vec4(a_position_i.xyz, 1.);
    int  i        = int(a_position_i.w);
#ifdef ATTRIB_MATRIX
    if      (i == 0) gl_Position = a_mat1 * position;
    else if (i == 1) gl_Position = a_mat2 * position;
    else if (i == 2) gl_Position = a_mat3 * position;
#else
    if      (i == 0) gl_Position = mat4(a_mat1_1, a_mat1_2, a_mat1_3, a_mat1_4) * position;
    else if (i == 1) gl_Position = mat4(a_mat2_1, a_mat2_2, a_mat2_3, a_mat2_4) * position;
    else if (i == 2) gl_Position = mat4(a_mat3_1, a_mat3_2, a_mat3_3, a_mat3_4) * position;
#endif
    else
    {
    #ifdef ANDROID
        int j = (i - 3 + u_umatsPI * gl_InstanceID) << 2;
        gl_Position = mat4(u_mats[j++], u_mats[j++], u_mats[j++], u_mats[j]) * position;
    #else
        gl_Position = u_mats[i - 3 + u_umatsPI * gl_InstanceID] * position;
    #endif
    }

    v_instanceID = gl_InstanceID;

#ifdef TYPE_TEXTURE
    v_uv = a_uv;
#endif
    
#ifdef TYPE_COLOR
    v_color = a_color;
#endif
    
#ifdef USE_LIGHT
#ifdef VOXEL
    int n   = int(a_normal);
#ifdef ANDROID
    float s = float(1 - ((n&1)<<1));
    n >>= 1;
    if      (n == 0) v_light = u_lightVecs[i + u_matsPI * gl_InstanceID].x;
    else if (n == 1) v_light = u_lightVecs[i + u_matsPI * gl_InstanceID].y;
    else if (n == 2) v_light = u_lightVecs[i + u_matsPI * gl_InstanceID].z;
    v_light = 1. - max(v_light * s, 0.);
#else
    v_light = 1. - max(u_lightVecs[i + u_matsPI * gl_InstanceID][n>>1] * float(1 - ((n&1)<<1)), 0.);
#endif
#else
    mediump float l = max(dot(u_lightVecs[i + u_matsPI * gl_InstanceID].xyz, a_normal) - 0.25, 0.) * 1.33333 - 1.;
    v_light = 1. - (l * l * l + 1.);
#endif
#endif
    
#ifdef USE_SHADOW
    position = u_shadowMat * gl_Position;
    v_shadow = position.xyz / position.w;
#endif
}
